@@ -1,7 +1,8 @@
 use {
     crate::{state::*, errors::*},
-    anchor_lang::{prelude::*},
+    anchor_lang::{prelude::*, solana_program::program::invoke_signed},
     anchor_spl::{token::{self, Token, TokenAccount, Mint, Transfer, FreezeAccount, Approve}},
+    mpl_token_metadata::instruction::freeze_delegated_account,
     vipers::assert_keys_eq
 };
 
@@ -30,7 +31,7 @@ pub struct ClaimCtx<'info> {
     token_program: Program<'info, Token>,
 }
 
-pub fn handler(ctx: Context<ClaimCtx>) -> ProgramResult {
+pub fn handler<'key, 'accounts, 'remaining, 'info>(ctx: Context<'key, 'accounts, 'remaining, 'info, ClaimCtx<'info>>) -> ProgramResult {
     let token_manager = &mut ctx.accounts.token_manager;
     token_manager.recipient_token_account = ctx.accounts.recipient_token_account.key();
     token_manager.state = TokenManagerState::Claimed as u8;
@@ -67,7 +68,7 @@ pub fn handler(ctx: Context<ClaimCtx>) -> ProgramResult {
 
     // if this is a managed token, this means we will revoke it at the end of life, so we need to delegate and freeze
     if token_manager.kind == TokenManagerKind::Managed as u8 {
-        // set account delegate of recipient token account to certificate PDA
+        // set account delegate of recipient token account to token manager PDA
         let cpi_accounts = Approve {
             to: ctx.accounts.recipient_token_account.to_account_info(),
             delegate: token_manager.to_account_info(),
@@ -86,6 +87,40 @@ pub fn handler(ctx: Context<ClaimCtx>) -> ProgramResult {
         let cpi_program = ctx.accounts.token_program.to_account_info();
         let cpi_context = CpiContext::new(cpi_program, cpi_accounts).with_signer(token_manager_signer);
         token::freeze_account(cpi_context)?;
+    } else if token_manager.kind == TokenManagerKind::Edition as u8 {
+        let remaining_accs = &mut ctx.remaining_accounts.iter();
+        let edition_info = next_account_info(remaining_accs)?;
+        let metadata_program = next_account_info(remaining_accs)?;
+
+        // edition will be validated by metadata_program
+        assert_keys_eq!(metadata_program.key, mpl_token_metadata::id());
+
+        // set account delegate of recipient token account to token manager PDA
+        let cpi_accounts = Approve {
+            to: ctx.accounts.recipient_token_account.to_account_info(),
+            delegate: token_manager.to_account_info(),
+            authority: ctx.accounts.recipient.to_account_info(),
+        };
+        let cpi_program = ctx.accounts.token_program.to_account_info();
+        let cpi_context = CpiContext::new(cpi_program, cpi_accounts);
+        token::approve(cpi_context, token_manager.amount)?;
+        
+        invoke_signed(
+            &freeze_delegated_account(
+                *metadata_program.key,
+                token_manager.key(),
+                ctx.accounts.recipient_token_account.key(),
+                *edition_info.key,
+                ctx.accounts.mint.key(),
+            ),
+            &vec![
+                token_manager.to_account_info(),
+                ctx.accounts.recipient_token_account.to_account_info(),
+                edition_info.to_account_info(),
+                ctx.accounts.mint.to_account_info(),
+            ],
+            &[token_manager_seeds],
+        )?;
     }
     return Ok(())
 }
