@@ -1,7 +1,9 @@
 use {
     crate::{state::*, errors::*},
-    anchor_lang::{prelude::*},
-    anchor_spl::{token::{self, Token, TokenAccount, Mint, Transfer, ThawAccount, CloseAccount}}
+    anchor_lang::{prelude::*, solana_program::program::invoke_signed},
+    anchor_spl::{token::{self, Token, TokenAccount, Mint, Transfer, ThawAccount, CloseAccount}},
+    mpl_token_metadata::instruction::freeze_delegated_account,
+    vipers::assert_keys_eq
 };
 
 #[derive(Accounts)]
@@ -36,7 +38,7 @@ pub struct InvalidateCtx<'info> {
     token_program: Program<'info, Token>,
 }
 
-pub fn handler(ctx: Context<InvalidateCtx>) -> ProgramResult {
+pub fn handler<'key, 'accounts, 'remaining, 'info>(ctx: Context<'key, 'accounts, 'remaining, 'info, InvalidateCtx<'info>>) -> ProgramResult {
     let token_manager = &mut ctx.accounts.token_manager;
 
     // get PDA seeds to sign with
@@ -58,6 +60,40 @@ pub fn handler(ctx: Context<InvalidateCtx>) -> ProgramResult {
             token::thaw_account(cpi_context)?;
 
         }
+        // transfer back to issuer
+        let cpi_accounts = Transfer {
+            from: ctx.accounts.recipient_token_account.to_account_info(),
+            to: ctx.accounts.issuer_token_account.to_account_info(),
+            authority: token_manager.to_account_info(),
+        };
+        let cpi_program = ctx.accounts.token_program.to_account_info();
+        let cpi_context = CpiContext::new(cpi_program, cpi_accounts).with_signer(token_manager_signer);
+        token::transfer(cpi_context, token_manager.amount)?;
+    } else if token_manager.kind == TokenManagerKind::Edition as u8 {
+        let remaining_accs = &mut ctx.remaining_accounts.iter();
+        let edition_info = next_account_info(remaining_accs)?;
+        let metadata_program = next_account_info(remaining_accs)?;
+
+        // edition will be validated by metadata_program
+        assert_keys_eq!(metadata_program.key, mpl_token_metadata::id());
+        
+        invoke_signed(
+            &freeze_delegated_account(
+                *metadata_program.key,
+                token_manager.key(),
+                ctx.accounts.recipient_token_account.key(),
+                *edition_info.key,
+                ctx.accounts.mint.key(),
+            ),
+            &vec![
+                token_manager.to_account_info(),
+                ctx.accounts.recipient_token_account.to_account_info(),
+                edition_info.to_account_info(),
+                ctx.accounts.mint.to_account_info(),
+            ],
+            &[token_manager_seeds],
+        )?;
+
         // transfer back to issuer
         let cpi_accounts = Transfer {
             from: ctx.accounts.recipient_token_account.to_account_info(),
